// -*- coding: utf-8 -*-

// Copyright (C) 2013 Stefan Besler <melongex@gmail.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


//! Initialize the plasmoid: Set global variables and listen to configuration changes
function initialize() {
  //  console.log("initializing...")
    plasmoid.setAspectRatioMode(IgnoreAspectRatio);
    plasmoid.globals = {} // Place to store global variables
    plasmoid.globals.steam_id = "";
    plasmoid.globals.steam_key = ""; 
    plasmoid.globals.player_name = "";
    plasmoid.globals.player_icon = "";
    plasmoid.globals.player_url = "";
    plasmoid.globals.player_account_id = "";
    plasmoid.globals.matches_requested = "25";
    plasmoid.globals.hero_map = new Object();

    // Listen for config changes
    plasmoid.addEventListener('ConfigChanged', configChanged)
}

//! This function is called whenever the config changes, or when the plasmoid is started
//! It will extract all important fields in the configuration dialog and fetch dota 2
//! data if the configuratino changed
function configChanged() {

  //  console.log("Config has changed");
    data.clear();

    polltimer.running = false;

    plasmoid.globals.steam_id = "";
    plasmoid.globals.steam_key = "";
    plasmoid.globals.player_name = "";
    player_name = "";
    player_icon = "";
    plasmoid.globals.player_icon = "";
    plasmoid.globals.player_url = "";
    plasmoid.globals.player_account_id = "";
    plasmoid.globals.matches_requested = "25";
    plasmoid.globals.hero_map = new Object();

    // Pause timer
    //polltimer.running = false

    // Read configuration
    plasmoid.globals["steam_id"] = String(plasmoid.readConfig("steam_id"))
    plasmoid.globals["steam_key"] = String(plasmoid.readConfig("steam_key"))

    if (!plasmoid.globals.steam_id) {
        // Update overlay
        setTooltip("Error", "Dotastat not configured.");
       // console.log("not configured");
    }
    else if (!plasmoid.globals.steam_key) {
        // Update overlay
        setTooltip("Error", "Dotastat not configured.");
       // console.log("not configured");
    }
    else
    {
        //console.log("querying player ...");
        queryPlayer();
        queryHeroMap();
        polltimer.interval = 15 * 60000;
        polltimer.running = true;
    }

   // console.log("config accepted");
  }

//! In the Dota2 Webapi each hero is represented by an unique hero id. This function will gather
//! the information that is need to associate such an ID with a hero name; i.e. 20 = Vengeful Spirit
function queryHeroMap(){
   // console.log("requesting map");
    var hmap = new Object;
    var request = new XMLHttpRequest();
    var hero_prefix = "npc_dota_hero_";
    var hero_prefix_len = hero_prefix.length;
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status ===200) {
            var document = request.responseXML.documentElement;
            for(var i = 0; i < document.childNodes.length; i++) {
                var result = document.childNodes[i];
                if(result.tagName === "heroes") {
                    for(var j = 0; j < result.childNodes.length; j++){
                        var record = result.childNodes[j];
                        if (record.tagName === 'hero'){
                            var hid = '';
                            var hname = '';
                            for (var k = 0; k < record.childNodes.length; k++){
                                var elem = record.childNodes[k];
                                if(elem.tagName === 'id'){
                                    hid = elem.childNodes[0].nodeValue;
                                }
                                else if(elem.tagName === 'name'){
                                    hname = elem.childNodes[0].nodeValue;
                                }
                            }
                            hmap[hid] = hname.substring(hero_prefix_len, hname.length);
                           // console.log(hid+ "->"+ hmap[hid]);
                        }
                    }
                    plasmoid.globals.hero_map = hmap;
                    queryMatches();
                }
            }
        }
        else
        {

        }
    }

    var steamurl= "https://api.steampowered.com/IEconDOTA2_570/GetHeroes/V1/?format=XML&key="+plasmoid.globals.steam_key;
 //   console.log("Requesting heroes: "+steamurl);
    request.open('GET', steamurl);
    request.send(null);
}

//! We use dotabuff to associate the player's steam id with his name and his account_id, which
//! is needed for the dota2 webapi. One could also calculate the players account_id from his
//! steam id - however, using dotabuff here is quite convenient
function queryPlayer() {
   //  console.log("requesting player information");
     plasmoid.globals.player_account_id = String(plasmoid.globals.steam_id.substr(3) - 61197960265728);

     var request = new XMLHttpRequest();
     request.onreadystatechange = function() {
         if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
             var document = request.responseXML.documentElement;
             var url = ""
             for(var i = 0; i < document.childNodes.length; i++) {
                 var result = document.childNodes[i];
                 if(result.tagName === "players") {
                     for(var j = 0; j < result.childNodes.length; j++){
                         var elem = result.childNodes[j];

                         if(elem.tagName === "player")
                         {
                       //      console.log("Found player tag");
                             for(var k = 0; k < elem.childNodes.length; k++){
                                 var player = elem.childNodes[k];
                              //   console.log(player.tagName);
                                 if(player.tagName === 'personaname')
                                 {
                                     //console.log("player name:"+player.childNodes[0].nodeValue);
                                     plasmoid.globals.player_name = player.childNodes[0].nodeValue;
                                     player_name = plasmoid.globals.player_name;

                                 }
                                 else if(player.tagName === 'profileurl'){
                                     plasmoid.globals.player_url = player.childNodes[0].nodeValue;
                                 }
                                 else if(player.tagName === 'avatar'){
                                     //console.log("player icon:"+player.childNodes[0].nodeValue);
                                     plasmoid.globals.player_icon = player.childNodes[0].nodeValue;
                                     player_icon = plasmoid.globals.player_icon;
                                 }
                             }
                         }
                     }
                 }
             }
         }
         else
         {

         }
     }

     var url = "http://api.steampowered.com/ISteamUser/GetPlayerSummaries/v0002/?key="+plasmoid.globals.steam_key+"&format=XML&steamids="+plasmoid.globals.steam_id;
     request.open('GET', url);
     request.send();

  //   console.log("GetPlayerSummaries: "+url);
}

//! This function fetches the players last matches from the dota2 webapi
function queryMatches() {
    //console.log("queryMatches")
    data.clear();
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseXML.documentElement;
            var dataServer = {
                match_id: '',
                hero_id: '',
                start_time: '',
                gold: '',
                last_hits: '',
                denies: '',
                gold_spent: '',
                kills: '',
                deaths: '',
                assists: '',
                duration: '',
                gpm: '',
                xpm: '',
                level: '',
                hero_damage: '',
                tower_damage: '',
                player_slot: '',
                radiant_win: '',
                skill: 1,
            };
            var id = 0;
            for(var i = 0; i < document.childNodes.length; i++) {
                var result = document.childNodes[i];
                if(result.tagName === "matches") {
                    for(var j = 0; j < result.childNodes.length; j++) {
                        var matches = result.childNodes[j];

                        if(matches.tagName === "match") {
                            for (var k = 0; k < matches.childNodes.length; k++){
                                var match = matches.childNodes[k];

                                if(match.tagName === 'match_id'){
                                    if(data.count <= id)
                                    {
                                        dataServer.match_id = match.childNodes[0].nodeValue;
                                        //console.log("Generating item with id: "+id+ " and matchid: "+dataServer.match_id)
                                        data.append(dataServer);
                                    }

                                    queryDetails(id);
                                    id = id + 1;
                                }
                            }
                        }
                    }
                }
            }
        }
        else
        {

        }
    }
    var url = "https://api.steampowered.com/IDOTA2Match_570/GetMatchHistory/V001/?account_id="+plasmoid.globals.player_account_id+"&format=XML&matches_requested="+plasmoid.globals.matches_requested+"&key="+plasmoid.globals.steam_key;
  //  console.log("requesting: "+url);
    request.open('GET', url);
    request.send();
}

//! This function extracts important information of a specific match
function queryDetails(id) {
    var currentId = id;
    var listElement = data.get(currentId);

    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200)
        {
            var document = request.responseXML.documentElement;

            for(var i = 0; i < document.childNodes.length; i++) {
                var record = document.childNodes[i];
                if (record.tagName === 'radiant_win') {
                    listElement.radiant_win = record.childNodes[0].nodeValue;
                }
                else if (record.tagName === 'match_id') {
                    listElement.match_id = record.childNodes[0].nodeValue;
                }
                else if (record.tagName === 'start_time') {
                    listElement.start_time = record.childNodes[0].nodeValue;
                }
                else if (record.tagName === 'players') {
                    for (var k = 0; k < record.childNodes.length; k++){
                        var elem = record.childNodes[k];
                        if(elem.tagName === 'player'){
                            for (var j = 0; j < elem.childNodes.length; j++){
                                var player = elem.childNodes[j];
                                if(player.tagName === "account_id")
                                {   // Wir haben uns gefunden, jetzt gehen wir alles durch und suchen die hero id in diesem player set

                                    var pid = player.childNodes[0].nodeValue;
                                   // console.log(pid + "-" + plasmoid.globals.player_account_id);
                                    if(pid === plasmoid.globals.player_account_id)
                                    {

                                       for (var z = 0; z < elem.childNodes.length; z++){
                                            player = elem.childNodes[z];
                                          //  console.log(player.tagName);
                                            if(player.tagName === "player_slot") {
                                                listElement.player_slot = player.childNodes[0].nodeValue;
                                            }
                                            if(player.tagName === "hero_id") {
                                                listElement.hero_id = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "kills") {
                                                listElement.kills = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "deaths") {
                                                listElement.deaths = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "assists") {
                                                listElement.assists = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "gold_per_min") {
                                                listElement.gpm = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "xp_per_min") {
                                                listElement.xpm = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "tower_damage") {
                                                listElement.tower_damage = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "hero_damage") {
                                                listElement.hero_damage = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "level") {
                                                listElement.level = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "last_hits") {
                                                listElement.last_hits = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "denies") {
                                                listElement.denies = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "gold") {
                                                listElement.gold = player.childNodes[0].nodeValue;
                                            }
                                            else if(player.tagName === "gold_spent") {
                                                listElement.gold_spent = player.childNodes[0].nodeValue;
                                            }
                                        }
                                    }

                                }
                            }
                        }
                    }
                }
                else if(record.tagName === "duration") {
                  //  console.log(record.childNodes[0].nodeValue)
                    listElement.duration = record.childNodes[0].nodeValue;
                }
            }

            querySkill(2);
            }
            else{
            }
       }


    var url = "https://api.steampowered.com/IDOTA2Match_570/GetMatchDetails/V001/?format=xml&match_id="+listElement.match_id+"&key="+plasmoid.globals.steam_key;
   // console.log("requesting: "+url)
    request.open('GET', url);
    request.send();
}

function querySkill(skill_level) {
   // console.log("querySkill for"+skill_level);
    var request = new XMLHttpRequest();
    request.onreadystatechange = function() {
        if (request.readyState === XMLHttpRequest.DONE && request.status === 200) {
            var document = request.responseXML.documentElement;

            var id = 0;
            for(var i = 0; i < document.childNodes.length; i++) {
                var result = document.childNodes[i];
                if(result.tagName === "matches") {
                    for(var j = 0; j < result.childNodes.length; j++) {
                        var matches = result.childNodes[j];

                        if(matches.tagName === "match") {

                            for (var k = 0; k < matches.childNodes.length; k++){
                                var match = matches.childNodes[k];

                                if(match.tagName === 'match_id'){
                                  //  console.log(match.childNodes[0].nodeValue);
                                    for(var h = 0; h<data.count; h++) {
                                        if(data.get(h).match_id === match.childNodes[0].nodeValue) {
                                            data.get(h).skill = parseInt(skill_level);
                                           // console.log("skill:"+data.get(h).skill);
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }

            if(skill_level < 3)
                querySkill(skill_level + 1);
        }
        else
        {

        }
    }
    var url = "https://api.steampowered.com/IDOTA2Match_570/GetMatchHistory/V001/?skill="+skill_level+"&account_id="+plasmoid.globals.player_account_id+"&format=XML&matches_requested="+plasmoid.globals.matches_requested+"&key="+plasmoid.globals.steam_key;
    //console.log("query+ "+url);
    request.open('GET', url);
    request.send();
}

//! Download the appropriate hero icon for a specific hero_id
function heroIcon(hid) {
if(!plasmoid.globals.hero_map[hid])
  return "";
return "http://media.steampowered.com/apps/dota2/images/heroes/"+plasmoid.globals.hero_map[hid]+"_sb.png"
}

//! The player won the game if he is on radiant side and radiant side wins, or if he is dire and radiant looses
//! This function return green if the player one and red otherwise
function textColor(hasRadiantWon, player_slot) {
    if((hasRadiantWon == "true" && parseInt(player_slot) < 128) || (hasRadiantWon == "false" && parseInt(player_slot)>=128))
        return "#33fc33";
    return "#fc3333";
}

//! How much time passed since this match?
function textWhen(startTime, duration) {
    var date = new Date();
    var ts = Math.round((new Date()).getTime() / 1000);

    var minutes = parseFloat(ts - parseInt(startTime) - parseInt(duration)) / 60;
    if(minutes < 60)
        return Math.round(minutes)+" min ago";

    var hours = minutes / 60;
    if(hours < 24)
        return Math.round(hours)+" hours ago";

    var days = hours / 24;
    return Math.round(days)+" days ago";
}

function textCaption(hasRadiantWon, player_slot) {
    if((hasRadiantWon == "true" && parseInt(player_slot) < 128) || (hasRadiantWon == "false" && parseInt(player_slot)>=128))
        return "Won match";
    return "Lost match";
}


function statusIcon(hasRadiantWon, player_slot) {
    if((hasRadiantWon == "true" && parseInt(player_slot) < 128) || (hasRadiantWon == "false" && parseInt(player_slot)>=128))
        return "flag-green";    
    return "flag-red";
}

function textSkill(skill_level) {
    if(skill_level === 1)
        return "Normal";
    else if(skill_level === 2)
        return "High";
    else if(skill_level === 3)
        return "Very high";
}

function textDuration(duration) {
    return Math.round(parseFloat(duration) / 60.0)+" m";
}

function textKda(kills, deaths, assists) {
    return ((Math.round(kills) + Math.round(assists)) / Math.round(deaths)).toFixed(2);
}
