// -*- coding: utf-8 -*-

// Copyright (C) 2013 Stefan Besler <melongex@gmail.com>
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License along
// with this program; if not, write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

import QtQuick 1.0
import org.kde.plasma.components 0.1 as PlasmaComponents
import org.kde.plasma.graphicswidgets 0.1 as PlasmaWidgets
import org.kde.plasma.core 0.1 as PlasmaCore
import "plasmapackage:/code/dotastat.js" as DS

Rectangle {
    id: wrapper
    color: "transparent"

    // Setup the plasmoid
    property int minimumWidth: 342
    property int minimumHeight: 250
    property int headerHeight: 40
    property int headerMargin: 5
    property string player_name: ""
    property string player_icon: ""
    property variant hero_map

    ListModel {
        id: data
    }



    Component.onCompleted: {
        DS.initialize();
    }


    Column {
        anchors.fill: parent
        anchors.centerIn: parent

        Rectangle {
            id: header
            width: listView.width - wrapper.headerMargin
            height: 30
            color: "transparent"
            Row {
                anchors {
                    fill: centerIn
                    centerIn: parent
                }

                Image {
                    width: header.height - 2
                    height: header.height - 2
                    fillMode: Image.PreserveAspectFit
                    smooth: true
                    source: wrapper.player_icon;
                }

                PlasmaComponents.Label {
                    height: header.height;
                    verticalAlignment: Qt.AlignVCenter
                    horizontalAlignment: Qt.AlignHCenter
                    font.capitalization: Font.AllUppercase
                    font.bold: true
                    font.weight: Font.DemiBold
                    //style: Text.Raised;
                    //styleColor: Qt.lighter(theme.textColor, 3)
                    text: " " + wrapper.player_name
                }
            }
        }

        ListView {
            id: listView
            width: wrapper.width
            height: wrapper.height - refreshButton.height - header.height


            Component {
                id: matchDelegate
                //Item {
                Item {
                    id: matchItem
                    width: wrapper.width - wrapper.headerMargin
                    height: wrapper.headerHeight + matchDetails.height
                    state: "normal"

                    Column {

                        // Hauptansicht
                        Rectangle {
                            id: item
                            width: matchItem.width
                            height: matchItem.height - wrapper.headerMargin - matchDetails.height
                            color: "transparent"

                            Rectangle {
                                id: itemBackground
                                width: matchItem.width
                                height: matchItem.height - wrapper.headerMargin - matchDetails.height
                                color: theme.backgroundColor
                                opacity: 0.2
                            }

                            Row {
                                anchors {
                                    fill: parent
                                    centerIn: parent
                                }
                                PlasmaComponents.Label {
                                    horizontalAlignment: Text.AlignLeft
                                    verticalAlignment: Text.AlignVCenter
                                    width: 5
                                }

                            // Bild des Helden
                                Image {
                                   // width: 39
                                    y: item.height * 0.1
                                    height: item.height * 0.8
                                    fillMode: Image.PreserveAspectFit
                                    smooth: true
                                    source: DS.heroIcon(hero_id)
                                }

                                PlasmaComponents.Label {
                                    horizontalAlignment: Text.AlignLeft
                                    verticalAlignment: Text.AlignVCenter
                                    width: 100
                                    height: item.height
                                    text: "  " + DS.textWhen(start_time, duration)
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }


                                PlasmaCore.IconItem {
                                    y: 5
                                    width: parent.height * 0.75
                                    height: parent.height * 0.75
                                    source: DS.statusIcon(radiant_win, player_slot)
                                }

                                /*PlasmaComponents.Label {
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    width: 50
                                    height: item.height
                                    text: DS.textCaption(radiant_win, player_slot)
                                    color: DS.textColor(radiant_win, player_slot)
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }*/

                                Column {
                                    width: 90
                                    height: item.height
                                    PlasmaComponents.Label {
                                        horizontalAlignment: Text.AlignHCenter
                                        verticalAlignment: Text.AlignVCenter
                                        width: 90
                                        height: item.height / 2
                                        text: "K/D/A"
                                        //style: Text.Raised;
                                        //styleColor: Qt.lighter(theme.textColor, 3)
                                    }

                                    PlasmaComponents.Label {
                                        horizontalAlignment: Text.AlignHCenter
                                        verticalAlignment: Text.AlignVCenter
                                        width: 90
                                        height: item.height / 2
                                        text: kills + "/" + deaths + "/" + assists
                                        //style: Text.Raised;
                                        //styleColor: Qt.lighter(theme.textColor, 3)
                                        }
                                }

                                Column
                                {
                                    // KDA visualisieren
                                    Rectangle {
                                        id: kdaRectangle
                                        y: parent.height / 4 + 3
                                        width: 60
                                        height: 3
                                        color: theme.linkColor
                                        property int kdaSum: kills*1 + assists*1 + deaths*1


                                        Row {

                                            anchors {
                                                fill: parent
                                                centerIn: parent
                                            }

                                            Rectangle {
                                                width: { return (kdaRectangle.width * kills / (kdaRectangle.kdaSum)); }
                                                height: kdaRectangle.height
                                                color: theme.visitedLinkColor

                                            }
                                            Rectangle {
                                                width: { return (kdaRectangle.width * deaths / (kdaRectangle.kdaSum));}
                                                height: kdaRectangle.height
                                                color: theme.buttonBackgroundColor
                                            }
                                            Rectangle {
                                                width: { return (kdaRectangle.width * assists / (kdaRectangle.kdaSum));}
                                                height: kdaRectangle.height
                                                color: theme.linkColor
                                            }

                                        }
                                    }


                                    PlasmaComponents.Label {
                                        horizontalAlignment: Text.AlignHCenter
                                        verticalAlignment: Text.AlignVCenter
                                        y: 3 * parent.height / 4 - 5
                                        width: 60
                                        height: item.height / 2
                                        text: DS.textKda(kills, deaths, assists)
                                        //style: Text.Raised;
                                        //styleColor: Qt.lighter(theme.textColor, 3)
                                    }
                                }

                                Rectangle {
                                    color:"transparent"
                                    width: 20
                                    height: item.height
                                }
/*
                                // Skill einzeichnen
                                Repeater {
                                    model: skill-1
                                    PlasmaCore.IconItem {
                                        height:  item.height / 1.8
                                        width: item.height / 1.8
                                        y: item.height / 2 - height / 2
                                        source: "bookmarks"
                                    }
                                }

                                Repeater {
                                    model: 3-skill
                                    PlasmaCore.IconItem {
                                        height:  item.height / 1.8
                                        width: item.height / 1.8
                                        y: item.height / 2 - height / 2
                                        source: "bookmarks"
                                        opacity: .3
                                    }
                                }
*/

                              /*  PlasmaComponents.Label {
                                    horizontalAlignment: Text.AlignHCenter
                                    verticalAlignment: Text.AlignVCenter
                                    width: 50
                                    height: item.height
                                    text: DS.textSkill(skill)
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }*/
                                }

                            // Zeile fÃ¼r Hauptansicht
                            /*Row{
                                anchors.verticalCenter: parent.verticalCenter
                                anchors.left: parent.left
                                anchors.leftMargin: 10
                                PlasmaComponents.Label {
                                    width: matchItem.width / 2
                                    horizontalAlignment: Qt.AlignCenter
                                    text: name
                                }
                                PlasmaComponents.Label {
                                    width: matchItem.width / 2
                                    horizontalAlignment: Qt.AlignCenter
                                    text: '$' + cost;
                                    visible: listView.currentIndex == index
                                }
                            }*/
                        }

                        // Platz für Detailinformationen
                        Rectangle{
                            id: matchDetails
                            x: 0
                            width: matchItem.width
                            height: 0
                            clip: true
                            color: "transparent"

                            property int matchDetailsMaxHeight: 95

                            Rectangle {
                                width: matchDetails.width
                                height: matchDetails.height
                                color: itemBackground.color
                                opacity: 0.15
                            }

                            // Grid Darstellung von Eigenschaften des Helden
                            Grid{
                                id: detailGrid
                                property color overlayColor: "transparent"
                                property int columnWidth: detailGrid.width / 4
                                columns: 4
                                rows: 5
                                anchors.fill: matchDetails
                                anchors.verticalCenter: matchDetails.verticalCenter
                                anchors.left: parent.left
                                anchors.top: parent.top
                                anchors.leftMargin: 10
                                anchors.topMargin: 10


                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "Level:"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: level
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "gpm"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: gpm
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "Gold spent"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: gold_spent
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "xpm:"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: xpm
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "Hero dmg:"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: hero_damage
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "Tower dmg:"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: tower_damage
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "LH/D:"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: last_hits + "/" + denies
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }

                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: "Duration:"
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                                PlasmaComponents.Label {
                                    width: detailGrid.columnWidth
                                    color: detailGrid.overlayColor
                                    text: DS.textDuration(duration)
                                    //style: Text.Raised;
                                    //styleColor: Qt.lighter(theme.textColor, 3)
                                }
                            }
                        }
                    }


                    // States
                    states: [
                        State {
                            name: "normal"
                            PropertyChanges { target: matchDetails; height: 0 }
                            //PropertyChanges { target: matchDetails; x: 0 }
                            PropertyChanges { target: matchDetails; width: matchItem.width }
                            PropertyChanges { target: detailGrid; overlayColor: "transparent" }
                            PropertyChanges { target: itemBackground; color: theme.backgroundColor }
                            },

                            State{
                                name: "detail"
                                PropertyChanges { target: matchDetails; height: matchDetails.matchDetailsMaxHeight }
                                PropertyChanges { target: matchDetails; width: matchItem.width - matchDetails.x }
                                PropertyChanges { target: detailGrid; overlayColor: theme.textColor }
                                PropertyChanges { target: itemBackground; color: Qt.darker(theme.backgroundColor) }
                            }
                         ]

                    // Transitions
                    transitions:
                        Transition {
                        from: "normal"; to: "detail"; reversible: true
                        ParallelAnimation {
                            NumberAnimation { properties: "height"; duration: 300; }
                            NumberAnimation { properties: "width"; duration: 300; }
                            ColorAnimation  { properties: "overlayColor"; duration: 500; }
                            ColorAnimation  { properties: "color"; duration: 500; }
                        }
                    }

                    // Elemente anklicken und highlighten
                    MouseArea {
                        //anchors.fill: parent
                        width: matchItem.width
                        height: wrapper.headerHeight
                        onEntered: { listView.currentIndex = index }
                        onClicked: {
                            if (matchItem.state === "normal")
                                matchItem.state = "detail"
                            else
                                matchItem.state = "normal"
                        }
                        onDoubleClicked: {
                            Qt.openUrlExternally("http://google.at")
                        }
                        hoverEnabled: true
                    }
                }
            }

            // Highlight
            Component {
                id: highlightDelegate
                PlasmaComponents.Highlight {
                    width: wrapper.width - wrapper.headerMargin
                    height: wrapper.headerHeight - wrapper.headerMargin
                    y: {
                        if(listView.currentItem)
                            return listView.currentItem.y;
                        return 0;
                    }
                    Behavior on y {
                        SpringAnimation {
                            spring: 2
                            damping: 0.5
                        }
                    }
                }
            }

            PlasmaCore.Theme {
                id: theme
            }

            Timer {
                id: polltimer
                running: false
                repeat: true
                onTriggered: { DS.queryMatches(); }
           }

            focus: true
            clip: true
            model: data
            delegate: matchDelegate
            highlightFollowsCurrentItem: false
            highlight: highlightDelegate
        }

        PlasmaComponents.Button {
            id: refreshButton
            text: "refresh"
            onClicked: { DS.queryMatches(); }
        }

    }

}

